@@ -5,7 +5,7 @@ from pyRDDLGym import ExampleManager
 from pyRDDLGym.Policies.Agents import RandomAgent
 # from pyRDDLGym.Visualizer.MovieGenerator import MovieGenerator
 
-def main(env, inst):
+def main(env, inst, method_name=None, episodes=1):
     print(f'preparing to launch instance {inst} of domain {env}...')
     
     # get the environment info
@@ -13,10 +13,13 @@ def main(env, inst):
     EnvInfo = ExampleManager.GetEnvInfo(env)
     
     # set up the environment class, choose instance 0 because every example has at least one example instance
+    log = False if method_name is None else True
     myEnv = RDDLEnv.RDDLEnv(domain=EnvInfo.get_domain(), 
                             instance=EnvInfo.get_instance(inst),
                             enforce_action_constraints=False,
-                            debug=False)
+                            debug=False,
+                            log=log,
+                            simlogname=method_name)
     
     # set up the environment visualizer
     # frames_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'Visualizer', 'Frames')
@@ -27,7 +30,7 @@ def main(env, inst):
     agent = RandomAgent(action_space=myEnv.action_space, 
                         num_actions=myEnv.numConcurrentActions)
 
-    for episode in range(1):
+    for episode in range(episodes):
         total_reward = 0
         state = myEnv.reset()
         for step in range(myEnv.horizon):
@@ -51,8 +54,18 @@ def main(env, inst):
 
 if __name__ == "__main__":
     args = sys.argv
+    method_name = None
+    episodes = 1
     if len(args) < 3:
         env, inst = 'HVAC', '0'
-    else:
+    elif len(args) < 4:
         env, inst = args[1:3]
-    main(env, inst)
+    elif len(args) < 5:
+        env, inst, method_name = args[1:4]
+    else:
+        env, inst, method_name, episodes = args[1:5]
+        try:
+            episodes = int(episodes)
+        except:
+            raise ValueError('episodes argument must be an integer, received:' + episodes)
+    main(env, inst, method_name, episodes)
