@@ -13,7 +13,7 @@ from pyRDDLGym.Core.Jax import JaxRDDLLogic
 from pyRDDLGym.Examples.ExampleManager import ExampleManager
 
 
-def get(path: str) -> Dict[str, object]:
+def get(path: str, **optional_args) -> Dict[str, object]:
     
     # read the config file
     path = os.path.join(os.path.dirname(os.path.abspath(__file__)), path)
@@ -26,18 +26,25 @@ def get(path: str) -> Dict[str, object]:
     
     # read the environment settings
     env_args = {k: args[k] for (k, v) in config.items('Environment')}
+    use_repo = env_args.get('check_rddlrepository', False)
+    if 'check_rddlrepository' in env_args:
+        del env_args['check_rddlrepository']
     
     # try to read from rddlrepository
     domain_name = env_args['domain']
     inst_name = env_args['instance']
     try:
-        print(f'reading domain {domain_name} from rddlrepository...')
+        if not use_repo:
+            raise Exception
+        warnings.warn(f'reading domain {domain_name} from rddlrepository...',
+                      stacklevel=2)
         from rddlrepository.Manager.RDDLRepoManager import RDDLRepoManager
         manager = RDDLRepoManager()
         EnvInfo = manager.get_problem(domain_name)
     except:
-        print(f'failed to read from rddlrepository, '
-              f'reading domain {domain_name} from Examples...')
+        warnings.warn(f'failed to read from rddlrepository, '
+                      f'reading domain {domain_name} from Examples...',
+                      stacklevel=2)
         EnvInfo = ExampleManager.GetEnvInfo(domain_name)
     
     env_args['domain'] = EnvInfo.get_domain()
@@ -68,15 +75,21 @@ def get(path: str) -> Dict[str, object]:
         opt_args['method_kwargs']['initializer'] = initializer
         if 'initializer_kwargs' in opt_args['method_kwargs']:
             del opt_args['method_kwargs']['initializer_kwargs']
+            
+    if 'method_kwargs' in opt_args and 'activation' in opt_args['method_kwargs']:
+        opt_args['method_kwargs']['activation'] = getattr(
+            jax.nn, opt_args['method_kwargs']['activation'])
+        
     opt_args['plan'] = getattr(JaxRDDLBackpropPlanner, opt_args['method'])(
         **opt_args['method_kwargs'])
-    opt_args['optimizer'] = getattr(optax, opt_args['optimizer'])(
-        **opt_args['optimizer_kwargs'])    
+    opt_args['optimizer'] = getattr(optax, opt_args['optimizer'])
     opt_args['logic'] = logic
     
     del opt_args['method']
     del opt_args['method_kwargs']
-    del opt_args['optimizer_kwargs']
+    if optional_args is not None:
+        for name, value in optional_args.items():
+            opt_args[name] = value
     optimizer = JaxRDDLBackpropPlanner.JaxRDDLBackpropPlanner(**opt_args)
     
     # read the train/test arguments
