@@ -8,7 +8,7 @@ from pyRDDLGym.Examples.ExampleManager import ExampleManager
 
 
 def test_xadd(
-        env_name: str = 'Wildfire',
+        env_name: str = 'wildfire',
         cpf: Optional[str] = None,
         save_graph: bool = False,
 ):
@@ -34,8 +34,14 @@ def test_xadd(
     xadd_model.compile()
     context = xadd_model._context
     
-    if cpf is not None:
-        expr = xadd_model.cpfs.get(f"{cpf}'")
+    if save_graph:
+        Path(f'tmp/{env_name}').mkdir(parents=True, exist_ok=True)
+        
+    if cpf is not None or cpf == 'reward':
+        if cpf == 'reward':
+            expr = xadd_model.reward
+        else:
+            expr = xadd_model.cpfs.get(f"{cpf}'")
         if expr is None:
             raise AttributeError(f"Cannot retrieve {cpf}' from 'model.cpfs'")
         print(f"cpf {cpf}':", end='\n')
@@ -59,7 +65,7 @@ if __name__ == "__main__":
 
     parser = argparse.ArgumentParser()
 
-    parser.add_argument('--env', type=str, default='Wildfire',
+    parser.add_argument('--env', type=str, default='wildfire',
                         help='The name of the RDDL environment')
     parser.add_argument('--cpf', type=str, default=None,
                         help='If specified, only print out this CPF')
